#include "StdAfx.h"
#include "SDHBackend.h"



SDHBackend::SDHBackend(void)
{
}

void SDHBackend::printout(Object ^o){
	printBuffer->Add(o);
}

void SDHBackend::init(ArrayList ^printBuffer){
	SDHBackend::printBuffer = printBuffer;
	printout("Backend initialised");
	options = new cSDHOptions();
	running = true;
	isFullAuto=false;
	iPressureThreshhold = 20;
	alpha_max = 5.0;
	beta_max = 90.0;
	fingerDelta = 60;	//Default value for rotation
	iGrabVelocity = 5;
	sensorHits= gcnew ArrayList();
	String ^time = (DateTime::Now).ToString();
	time = time->Replace(':','.');
	time = time->Replace(' ','.');
	printout("Time: "+time);
	tw = gcnew StreamWriter("data/"+time+".txt");
	printout("Created file data/"+time+".txt");
	
	//tw->WriteLine("Initialising textwriter");
	//tw->Close();
	sensorMax = gcnew ArrayList();
	for(int i=0;i<6;i++)
		sensorMax->Add(0);
}

double SDHBackend::deg2rad(double rad){
	return rad*180.0/Math::PI;
}

void SDHBackend::connect(){
	printout("Connecting to hand");
	if(hand!=NULL){
		if(hand->IsOpen()){
			printout("Error: Connection to hand already open!");
			return;
		}
	}
	try{
		hand = new cSDH();
		hand->OpenRS232( options->sdhport, options->rs232_baudrate, options->timeout, options->sdh_rs_device );
		if(hand->IsOpen()) printout("Connected to SDH hardware!");
		
		dsa = new cDSA(0,1,options->dsa_rs_device);
		dsa->SetFramerate( 1);
		/*
		printout(dsa->GetControllerInfo().ToString());
		printout(dsa->GetSensorInfo());
		printout(dsa->GetMatrixInfo(options->matrixinfo));
		*/

	} catch (Exception ^ex){
		printout("Error connecting to SDH hardware: "+ex);
	}
}

void SDHBackend::clearTargets(){

	try{
		std::vector<double> angles = hand->GetAxisActualAngle(hand->all_axes);
		hand->SetAxisTargetAngle(hand->all_axes,angles);
	}catch (Exception ^e){
		printout("Error: "+e->ToString());
	}
}

void SDHBackend::openHandStart(){
	openHandThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::openHand));
	openHandThread->Start();
}

void SDHBackend::openHand(){
	fingerDelta=60.0;
	if(hand==NULL){
		printout("Error: connection to SDH not estabelished");
		return;
	}
	if(!hand->IsOpen()){
		printout("Error: connection to SDH not estabelished");
		return;
	}
	
	//Clear all hits, starting over
	sensorHits->Clear();

	printout("Opening hand!");
	//This can be a bit faster
	hand->SetAxisTargetVelocity(hand->All,40.0);

	//Rotation
	hand->SetAxisTargetAngle(0,fingerDelta);
	
	//Proximal angles : -70 deg, distal: 0
	hand->SetAxisTargetAngle(1,-70.0);
	hand->SetAxisTargetAngle(2,0.0);
	hand->SetAxisTargetAngle(3,-70.0);
	hand->SetAxisTargetAngle(4,0.0);
	hand->SetAxisTargetAngle(5,-70.0);
	hand->SetAxisTargetAngle(6,0.0);

	//Do the movement
	
	try{
		moving = true;
		hand->MoveHand();
		moving = false;
	} catch (Exception ^e){
		hand->Stop();
		moving = false;
		printout("Error moving hand: "+e->ToString());
		return;
	}

	//Disable the motors
	hand->SetFingerEnable(hand->All,false);
	
	//Kill all threads
/*	try{
		graspingThread->Abort();
	}catch (Exception ^){
	}*/
	try{
		openHandThread->Abort();
	}catch (Exception ^){
	}
	try{
		closeGraspThread->Abort();
	}catch (Exception ^){
	}
	try{
		proximalsThread->Abort();
	}catch (Exception ^){
	}
	try{
		distalsThread->Abort();
	}catch (Exception ^){
	}	
	//printout("Hand opening complete! Motors disabled");
}

void SDHBackend::printSensors(){
	dsa->UpdateFrame();
	cDSA::sMatrixInfo matrixInfo[6];
	cDSA::tTexel texel;
	printout("Sensorpolling initialised");
	for(int m=0;m<6;m++){
		matrixInfo[m] = dsa->GetMatrixInfo(m);
		for(int y=0; y<matrixInfo[m].cells_y;y++){
			for(int x=0; x<matrixInfo[m].cells_x;x++){
				texel = dsa->GetTexel(m,x,y);
				printout("("+m+","+x+","+y+"): "+(int)texel);
			}
		}
	}
}

int SDHBackend::getMaxPressure(){
	String ^dataLogStr="";
	int iSensorMax;	//Max per sensor
	if(!running) return 0;
	try{
		dsa->UpdateFrame();
	} catch(Exception ^e){
		printout("Error reading pressure: "+e->ToString());
		return 0;
	}
	cDSA::sContactInfo contactInfo;
	cDSA::sMatrixInfo matrixInfo[6];
	int iMax = dsa->GetTexel(0,0,0);
	int iVal;
	cDSA::tTexel texel;
	for(int m=0;m<6;m++){
		iSensorMax=0;
		contactInfo = dsa->GetContactInfo(m);
		/*
		printout("Contactinfo for "+m+" (f, a, x, y) : " +contactInfo.force+" "
			+contactInfo.area+" "
			+contactInfo.cog_x+" "
			+contactInfo.cog_y);
		*/
		//printout("Pressure Point "+m+": " +dsa->GetContactArea(m));
		matrixInfo[m] = dsa->GetMatrixInfo(m);
		for(int y=0; y<matrixInfo[m].cells_y;y++){
			for(int x=0; x<matrixInfo[m].cells_x;x++){
				texel = dsa->GetTexel(m,x,y);
				iVal = (int)texel;
				if(iVal>iSensorMax) iSensorMax=iVal;
				if(iVal>iMax) iMax = iVal;
				if(iVal>iPressureThreshhold){
					
					if(noHits){		//Dont bother printing if we're already stopped	
						//Lets only count this hit if we dont have any hits on the same sensor from before!
						int iHitCount = sensorHits->Count;
						bool hasHitAlready = false;
						for(int i=0;i<iHitCount;i++){
							SensorHit ^hit = (SensorHit ^)sensorHits[i];
							if(hit->sensor == m){
								hasHitAlready= true;
							}
						}
						if(!hasHitAlready){
							printout("Pressure threshhold surpassed: " +iVal+" vs "+iPressureThreshhold);
							printout("Hit was in sensor "+m+" x:"+x+" y:"+y);
						
							noHits=false;
							lastHit = gcnew SensorHit(m,x,y,iVal);
							//printout("X comp:"+getGlobalXFromSensorHit(lastHit));
														
							sensorHits->Add(gcnew SensorHit(m,x,y,iVal));
						}
					}
				}

			}
		}
		dataLogStr+=iSensorMax+" ";
		sensorMax[m] = iSensorMax;
	}
	tw->WriteLine(dataLogStr);
	//printout("Max pressure: "+iMax);
	iMaxPressure = iMax;	



	return iMax;
}

void SDHBackend::shutDown(){
	running=false;
	if(dsa!=NULL) dsa->Close();
	if(hand!=NULL){
		hand->SetFingerEnable(hand->All,false);
		hand->Close();
	}
	tw->Close();
}

//Initialise separate thread for grasp-searching - clears previous hits!
void SDHBackend::graspSearchStart(){
	if(hand==NULL){
		printout("Error: connection to SDH not estabelished");
		return;
	}
	if(!hand->IsOpen()){
		printout("Error: connection to SDH not estabelished");
		return;
	}
	sensorHits->Clear();
	graspingThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::graspSearch ));
	noHits=true;
	graspingThread->Start();
}

void SDHBackend::graspSearch(){
	printout("Grasp Search commenced - old hits cleared!");
	//Open hand in seq mode (since we're already in a separate thread)
	openHand();
	sensorHits->Clear();
	printout("Initializing inwards search");
	closeGraspThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeGrasp));
	closeGraspThread->Start();
	moving = true;
	while(noHits && running){
		if(!moving) {
			printout("Movement complete");
			return;
		}
	}
	closeGraspThread->Abort();
	printout("Hit detected, halting search!");
	
	hand->Stop();
	clearTargets();
	//hand->SetFingerEnable(hand->All,false);
	calculateCentroid();
	//Should we disable motors or leave them on for holding on?
	
}
//Closes grasp for ALL fingers!
void SDHBackend::closeGrasp(){
	//Max angle for proximal joints. No egg will fit!
	hand->SetAxisTargetVelocity(hand->All,iGrabVelocity); //We want this to be quite slow
	hand->SetAxisTargetAngle(0,fingerDelta); //Lets just keep 60 deg (for a likesided tri)
	
	//Proximal joints target: alpha_max, distal: 0
	hand->SetAxisTargetAngle(1,alpha_max);
	hand->SetAxisTargetAngle(2,0.0);
	hand->SetAxisTargetAngle(3,alpha_max);
	hand->SetAxisTargetAngle(4,0.0);
	hand->SetAxisTargetAngle(5,alpha_max);
	hand->SetAxisTargetAngle(6,0.0);
	
	hand->MoveHand();
	
	// ^^ is blocking, so if we reach here, we've reached alpha max without hits
	// so lets power down motors
	moving = false;
	hand->SetFingerEnable(hand->All,false);
	printout("Search returned no results");
	state = "STATE_NOHITS";
}

int SDHBackend::getContactingFingers(){
	int finger0, finger1, finger2=0;
	for(int i =0; i<sensorHits->Count;i++){
		SensorHit ^sh = (SensorHit ^)(sensorHits[i]);
		int sensor = sh->sensor;
		if(sensor==0 || sensor==1) finger0=1;
		if(sensor==2 || sensor==3) finger1=1;
		if(sensor==4 || sensor==5) finger2=1;
	}

	return finger0 + finger1 + finger2;

}

void SDHBackend::manualTriggerHit(){
	noHits = false;
}

void SDHBackend::closeProximalsStart(){
	proximalsThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeProximals));
	proximalsThread->Start();
	noHits=true;
	moving = true;
	proximalsDetected= false;
	bool proximalsRunning = true;
	while(proximalsRunning && running && noHits){
		if(!moving && !isFullAuto) return;

	}
	proximalsThread->Abort();
	printout("Hit detected, halting proximals!");
	hand->Stop();
	//hand->SetFingerEnable(hand->All,false);
	clearTargets();
	/*
	//FULL AUTO MODE
	while(isFullAuto && activeAxis>0){
		proximalsThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeProximals));		
		proximalsThread->Start();
		noHits=true;
		moving = true;
		proximalsDetected= false;
		bool proximalsRunning = true;
		while(proximalsRunning && running && noHits){
			if(!moving && !isFullAuto) return;

		}
		proximalsThread->Abort();
		printout("Hit detected, halting proximals!");
		hand->Stop();
		clearTargets();
	}
	if(isFullAuto){
		printout("All proximals completed, now closing distals");
		Thread ^distalThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeDistalsStart));
		distalThread->Start();
	}
	*/

}

void SDHBackend::closeProximals(){
	clearTargets();
	inhibitAxis1, inhibitAxis3, inhibitAxis5 = false;
	SensorHit ^hit;
	int sensor;
	int iHitCount = sensorHits->Count;
	printout("Closing proximals: have "+iHitCount+" hits to respect:");
	for(int i=0;i<iHitCount;i++){
		hit = (SensorHit^)sensorHits[i];
		printout(hit->getString());
		sensor = hit->sensor;
		if(sensor == 0 || sensor == 1){
			inhibitAxis1 = true;
			printout("Axis 1 inhibited!");
		}
		if(sensor == 2 || sensor == 3){
			inhibitAxis3 = true;
			printout("Axis 3 inhibited!");
		}
		if(sensor == 4 || sensor == 5){
			inhibitAxis5 = true;
			printout("Axis 5 inhibited!");
		}
	}
	/*
	We want to move the fingers that dont have any hits on them.
	axis 1 has sensors 01
	axis 3 has sensors 23
	axis 5 has sensors 45
	*/
	try{
		hand->SetAxisTargetVelocity(hand->All,iGrabVelocity);
	} catch (Exception ^e){
		printout("Error setting velocity: "+e->ToString());
		return;
	}
	activeAxis=0;
	if(!inhibitAxis1){
		activeAxis++;
		hand->SetAxisTargetAngle(1,alpha_max);
	}
	if(!inhibitAxis3){
		activeAxis++;
		hand->SetAxisTargetAngle(3,alpha_max);
	}
	if(!inhibitAxis5){
		activeAxis++;
		hand->SetAxisTargetAngle(5,alpha_max);
	}
	
	proximalsDetected= true;
	try{
		hand->MoveHand();
		state = "STATE_PROXIMALSCOMPLETE";
	} catch (Exception ^e){
		printout("Error closing proximals: "+e->ToString());
	}
	moving = false;
	printout("Proximal closing complete, active axis: "+activeAxis);
	//hand->SetFingerEnable(hand->All,false);
	if(activeAxis==0){
		state="STATE_PROXIMALSCOMPLETE";
	}
	
}

void SDHBackend::closeDistalsStart(){
	printout("Starting to close distals");
	distalsThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeDistals));
	distalsThread->Start();
	noHits=true;
	moving = true;
	while(noHits && running){
		if(!moving) return;
	}
	distalsThread->Abort();
	printout("Hit detected, halting distals!");
	hand->Stop();
	//hand->SetFingerEnable(hand->All,false);
	clearTargets();
	
	/*
	//FULL AUTO MODE
	while(isFullAuto && activeAxis>0){
		distalsThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeDistals));
		distalsThread->Start();
		noHits=true;
		moving = true;
		while(noHits && running){
			if(!moving) return;
		}
		distalsThread->Abort();
		printout("Hit detected, halting distals!");
		hand->Stop();
		clearTargets();
	}
	if(isFullAuto){
		printout("All distals completed!");
		printout("GRIP COMPLETE!");
	}
	*/

}

void SDHBackend::closeDistals(){
	clearTargets();
	bool inhibitAxis2, inhibitAxis4, inhibitAxis6 = false;
	SensorHit ^hit;
	int sensor;
	int iHitCount = sensorHits->Count;
	printout("Closing distals: have "+iHitCount+" hits to respect:");
	for(int i=0;i<iHitCount;i++){
		hit = (SensorHit^)sensorHits[i];
		printout(hit->getString());
		sensor = hit->sensor;
		if(sensor == 1) inhibitAxis2 = true;
		if(sensor == 3) inhibitAxis4 = true;
		if(sensor == 5) inhibitAxis6 = true;
	}
	/*
	We want to move the distals that dont have any hits on them.
	axis 2 has sensors 1
	axis 4 has sensors 3
	axis 6 has sensors 5
	*/
	hand->SetAxisTargetVelocity(hand->All,iGrabVelocity);
	activeAxis=0;
	if(!inhibitAxis2){
		activeAxis++;
		hand->SetAxisTargetAngle(2,beta_max);
	}
	if(!inhibitAxis4){
		activeAxis++;
		hand->SetAxisTargetAngle(4,beta_max);
	}
	if(!inhibitAxis6){
		activeAxis++;
		hand->SetAxisTargetAngle(6,beta_max);
	}
	
	try{
		hand->MoveHand();
	} catch (Exception ^e){
		printout("Error moving distals: "+e->ToString());
		return;
	}
	//hand->SetFingerEnable(hand->All,false);
	printout("Distal closing complete");
	state = "STATE_DISTALSCOMPLETE";
	moving = false;
	
}

int SDHBackend::getHandAxisFromSensor(int sensor){
	int axis;
	if(sensor==0 || sensor == 1)
		axis = 1;
	if(sensor==2 || sensor ==3)
		axis = 3;
	if(sensor==4 || sensor ==5)
		axis = 5;
	return axis;

}

int SDHBackend::getHandAxisFromSensor(SensorHit ^sensorHit){
	return getHandAxisFromSensor(sensorHit->sensor);
}

int SDHBackend::getHitDistalPosition(SensorHit ^sensorHit){
	int sensor = sensorHit->sensor;
	int y = sensorHit->y;
		
	//y=0 -> distal direction
	//double angle = angles[axis];
	int isDistal = sensor%2;	//proximal; 0, distal: 1;
	
	cDSA::sMatrixInfo matrixInfo = dsa->GetMatrixInfo(sensor);
	int iSensorLength = matrixInfo.cells_y;
	//TODO: find detailed dims
	int proximalOffset=20;	//From palm to proximal
	int distalOffset=100;		//From palm to beginning of distal
	int sensorOffset= (iSensorLength-y)*0.75;
	int iHitLength= (proximalOffset+sensorOffset)*(1-isDistal)+(distalOffset+sensorOffset)*isDistal;
	return iHitLength;
	
}

int SDHBackend::getGlobalXFromSensorHit(SensorHit ^sensorHit){
	int axis = getHandAxisFromSensor(sensorHit);
	int dist = getHitDistalPosition(sensorHit);
	double alpha = hand->GetAxisActualAngle(axis);
	double beta = hand->GetAxisActualAngle(0);
	int x=0;
	int h=0;
	if(axis==1){	//- y
		//x = dist*cos(deg2rad(90+alpha))*cos(deg2rad(beta));
		//x = dist*cos(deg2rad(beta))*sin(deg2rad(alpha))*-1;
		x = dist*cos(deg2rad(beta));
	}
	else if(axis==3){	//Thumb
		//x = -dist*cos(deg2rad(90+alpha));
		//x = -dist*sin(deg2rad(alpha))*-1;
		x = -dist*sin(deg2rad(alpha));
	}
	else if(axis==5){ // +y
		//x = dist*cos(deg2rad(90+alpha))*cos(deg2rad(beta));
		x = dist*cos(deg2rad(beta));
	}
	printout("X comp axis: "+axis+" distance: "+dist+" alpha: "+alpha+" xcomp:"+x);
	return x;
}

int SDHBackend::getGlobalYFromSensorHit(SensorHit ^sensorHit){
	int axis = getHandAxisFromSensor(sensorHit);
	int dist = getHitDistalPosition(sensorHit);
	double alpha = hand->GetAxisActualAngle(axis);
	double beta = hand->GetAxisActualAngle(0);
	int y=0;
	int h=0;
	if(axis==1){	//- y
		//y = -dist*cos(deg2rad(90+alpha))*sin(deg2rad(beta));
		//y = -dist*sin(deg2rad(beta))*sin(deg2rad(alpha))*-1;
		y = -dist*sin(deg2rad(beta));
		
	}
	else if(axis==3){	//Thumb
		y = 0;
	}
	else if(axis==5){ // +y
		//y = dist*cos(deg2rad(90+alpha))*sin(deg2rad(beta));
		//y = dist*sin(deg2rad(beta))*sin(deg2rad(alpha))*-1;
		y = dist*sin(deg2rad(beta));
	}
	printout("Y comp axis: "+axis+" distance: "+dist+" alpha: "+alpha+" ycomp:"+y);
	return y;
}

void SDHBackend::calculateCentroid(){
	printout("Calculating Centroid");
	vector<double> angles = hand->GetAxisActualAngle(hand->all_real_axes);
	/*for(int i=0;i<angles.size();i++){
		printout(i+": "+angles[i]);
	}*/
	int x = lastHit->x;
	int y = lastHit->y;
	int sensor = lastHit->sensor;
	cDSA::sMatrixInfo matrixInfo = dsa->GetMatrixInfo(sensor);
	/*
		int axis;
	if(sensor==0 || sensor == 1)
		axis = 1;
	if(sensor==2 || sensor ==3)
		axis = 3;
	if(sensor==4 || sensor ==5)
		axis = 5;
	//y=0 -> distal direction
	double angle = angles[axis];
	int isDistal = sensor%2;	//proximal; 0, distal: 1;
	
	
	int iSensorLength = matrixInfo.cells_y;
	//TODO: find detailed dims
	int proximalOffset=20;	//From palm to proximal
	int distalOffset=100;		//From palm to beginning of distal
	int sensorOffset= (iSensorLength-y)*0.75;
	int iHitLength= (proximalOffset+sensorOffset)*(1-isDistal)+(distalOffset+sensorOffset)*isDistal;
	*/

	int iHitLength = getHitDistalPosition(lastHit);
	
	//even: proximal, odd: distal

	/* Bogus, lets use states instead
	if(MessageBox::Show("Is Centroid OK? Sensor: "+sensor+", x: "+x+",y: "+y+" ca. mm: "+iHitLength, "Yes it is!", MessageBoxButtons::YesNo) == DialogResult::Yes){
		printout("Centroid was OK, proceeding with closing fingers!");
		if(isFullAuto){
			Thread ^proximalsThread = gcnew Thread(gcnew ThreadStart(this, &SDHBackend::closeProximalsStart));
			proximalsThread->Start();
		}
	}else{
		printout("Centroid out of reach... ");
	}
	*/

	
	//Test if out of reach
	
	//tell state machine to return message to daddy - STATUS_OUTOFREACH
	
	
	//Test if hit x is close to 0 or cells_x.
	printout("Testing hit lateral location: hit was in "+x+", max is "+matrixInfo.cells_x);
	if(x==0 || x+1>=matrixInfo.cells_x){
		state = "STATE_NEEDROTATE";
		return;
	}

	//We're good homie, lets get on with the proximals!
		//Tell state machine to start proximals STATUS_CENTROIDOK
		state = "STATE_CENTROIDOK";

	
}

//Starts a thread that attempts to improve a detected contact. This one should not rely on collisions
//to be halted, but we need to make sure we don't go beyond max/min of fingers.
void SDHBackend::rotateFingersStart(){
	printout("Commencing rotation");
	double dAngleMin = 0;
	double dAngleMax = 90.0;

	//Assuming we're still usomg lastHit as the active hit.
	//rotation = K*distance
	int x = lastHit->x;
	int iHitLength = getHitDistalPosition(lastHit);
	double rot = -(10.0-(10.0*iHitLength)/160);
	int axis=1;
	double currRot = hand->GetAxisActualAngle(0);
	int sensor = lastHit->sensor;
	if(sensor == 0 || sensor == 1){ //axis 1
		if(x != 0) rot*= -1;
		axis = 1;
	}else if(sensor == 4 || sensor == 5){ //axis 5
		if(x==0) rot *= -1;
		axis = 5;
	}else{
		printout("Hit was on the thumb (sensor "+sensor+")");
		state = "STATE_CENTROIDOK";
		return;
	}
	//Less rotation the larger hitLength is
	double totalRot = currRot+rot;
	
	if(totalRot>dAngleMax) totalRot = dAngleMax;
	if(totalRot<dAngleMin) totalRot = dAngleMin;
	hand->SetAxisTargetVelocity(0,10.0);
	hand->SetAxisTargetVelocity(axis,10.0);
	hand->SetAxisTargetAngle(axis,hand->GetAxisTargetAngle(axis)-3.0);	// move out
	hand->MoveHand();
	sensorHits->Clear();
	hand->SetAxisTargetAngle(0,totalRot);								//rotate
	hand->MoveHand();
	hand->SetAxisTargetAngle(axis,hand->GetAxisTargetAngle(axis)+3.0);	//move in
	hand->MoveHand();
	hand->SetAxisTargetVelocity(hand->All,5.0);
	printout("Rotation complete");
	state = "STATE_CENTROIDOK";
	//Open up, 1 deg?, rotate, close 1 deg, state-> STATE_CENTROIDOK

}

void SDHBackend::startStateMachine(){
	//lets make a blocking - single run, triggered by a call from daddy

	//Start with the grasp search?
	state="STATE_SEARCHING";
	sensorHits->Clear();
	graspSearchStart();
	while(state=="STATE_SEARCHING"){
		//wait
	}

	//check the status set by calcCentroid
	if(state=="STATE_OUTOFREACH"){
		printout("State: out of reach");
		
		//Abort grasp
		//return message to daddy
	}
	if(state=="STATE_NEEDROTATE"){
		printout("state: need some rotating");
		rotateFingersStart();
		//rotate Fingers according to lastHit - lets make this blocking!
	}
	if(state=="STATE_CENTROIDOK"){	//assume this is correct
		printout("state: centroid is within reach");
		closeProximalsStart();
		//closeProximalsStart -  blocking
		while(state != "STATE_PROXIMALSCOMPLETE"){
			//-> closeProximalsStart - blocking
			Sleep(300);
			closeProximalsStart();
		}
	}
	if(state=="STATE_PROXIMALSCOMPLETE"){
		printout("state: proximals are done closing");
		//Test how many fingers have reahced their targets
		
		if(getContactingFingers()<2){
			state = "STATE_OUTOFREACH";
			int x = getGlobalXFromSensorHit(lastHit);
			int y = getGlobalYFromSensorHit(lastHit);

			printout("Object is out of reach! Need to move gripper (x,y) : ("+x+","+y+")");
			//calculate needed movement
			return;
		}
		closeDistalsStart(); // - blocking
		while(state!="STATE_DISTALSCOMPLETE"){
			//closeDistals - blocking
			Sleep(300);
			closeDistalsStart();
		}

	}

	if(state=="STATE_NOHITS"){
		printout("state: Grasp found nothing");
		//return message to daddy: no hits
	}
	
	if(state=="STATE_FIXROTATION"){
		printout("state: Found target, but need gripper rotation!");
	}

	if(state=="STATE_DISTALSCOMPLETE"){
		printout("state: Grip is complete!");
		//hand->SetFingerEnable(hand->All,false);
		//return message to daddy: grip complete
	}


}
